#define LGFX_USE_V1
#include <LovyanGFX.hpp>
#include <Wire.h>
#include <SD.h>
#include <FS.h>
#include <esp_camera.h>
#include <driver/i2s.h>
#define CAMERA_MODEL_ELECROW
#include "camera_pins.h"
int stream_flag = 1;

#define SD_MOSI 13
#define SD_MISO 14
#define SD_SCK 12
#define SD_CS 10

const uint8_t img_rgb888_480_320_head[54] = {
  0x42, 0x4d, 0xa0, 0x5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x36, 0x0, 0x0, 0x0, 0x28, 0x0,
  0x0, 0x0, 0xe0, 0x1, 0x0, 0x0, 0x40, 0x1, 0x0, 0x0, 0x1, 0x0, 0x18, 0x0, 0x0, 0x0,
  0x0, 0x0, 0x0, 0x5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
  0x0, 0x0, 0x0, 0x0, 0x0, 0x0
};
#define LCD_MOSI 13
#define LCD_MISO 14 // 12
#define LCD_SCK 12
#define LCD_CS 3
#define LCD_RST -1 // 26
#define LCD_DC 42  // 33
#define LCD_BL 46
class LGFX : public lgfx::LGFX_Device
{
    lgfx::Panel_ILI9488 _panel_instance;
    lgfx::Bus_SPI _bus_instance;

  public:
    LGFX(void)
    {
      {
        auto cfg = _bus_instance.config();
        cfg.spi_host = SPI3_HOST;
        cfg.spi_mode = 0;
        cfg.freq_write = 60000000;
        cfg.freq_read = 16000000;
        cfg.spi_3wire = true;
        cfg.use_lock = true;
        cfg.dma_channel = SPI_DMA_CH_AUTO;
        cfg.pin_sclk = LCD_SCK;
        cfg.pin_mosi = LCD_MOSI;
        cfg.pin_miso = LCD_MISO;
        cfg.pin_dc = LCD_DC;
        _bus_instance.config(cfg);
        _panel_instance.setBus(&_bus_instance);
      }

      {
        auto cfg = _panel_instance.config();

        cfg.pin_cs = LCD_CS;
        cfg.pin_rst = LCD_RST;
        cfg.pin_busy = -1;
        cfg.memory_width = 320;
        cfg.memory_height = 480;
        cfg.panel_width = 320;
        cfg.panel_height = 480;
        cfg.offset_x = 0;
        cfg.offset_y = 0;
        cfg.offset_rotation = 0;
        cfg.dummy_read_pixel = 8;
        cfg.dummy_read_bits = 1;
        cfg.readable = true;
        cfg.invert = false;
        cfg.rgb_order = false;
        cfg.dlen_16bit = false;
        cfg.bus_shared = true;
        _panel_instance.config(cfg);
      }
      setPanel(&_panel_instance);
    }
};
SPIClass SD_SPI;
File photo_file;
String imgname = "logo.bmp";
int img_index = 0;
LGFX lcd;
void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);

  pinMode(19, INPUT);

  sd_init();

  //LCD初始化
  lcd.init();
  lcd.setTextFont(4);
  lcd.setRotation( 3 ); /* Landscape orientation, flipped */
  pinMode(LCD_BL, OUTPUT);
  lcd.fillScreen(TFT_BLACK);
  delay(500);
  digitalWrite(LCD_BL, HIGH);



  camera_init();
  //  lcd.printf("Setup done");

}

void loop() {
  camera_fb_t *fb = NULL;
  fb = esp_camera_fb_get();
  if (stream_flag == 1)
  {
    //    lcd.pushImage(78, 40, fb->width, fb->height, (lgfx::swap565_t *)fb->buf);
    lcd.pushImage(0, 0, fb->width, fb->height, (lgfx::swap565_t *)fb->buf);
  }
  //  int pin40State = digitalRead(19);
  //  if (pin40State == HIGH) {
  //    Serial.println("IO40 is HIGH");
  //    void *ptrVal = NULL;
  //      ptrVal = heap_caps_malloc(480 * 320 * 3, MALLOC_CAP_SPIRAM);
  //      uint8_t *rgb = (uint8_t *)ptrVal;
  //      fmt2rgb888(fb->buf, fb->len, PIXFORMAT_RGB565, rgb);
  //      if (save_image(SD, rgb) == -1)
  //      {
  //        Serial.println("PHOTO Failed");
  //      }
  //      else
  //        Serial.println("PHOTO Success");
  //      heap_caps_free(ptrVal);
  //      rgb = NULL;
  //      delay(50);
  //  } else {
  //    Serial.println("IO40 is LOW");
  //  }
  esp_camera_fb_return(fb);

}

int sd_init()
{
  SD_SPI.begin(SD_SCK, SD_MISO, SD_MOSI);
  if (!SD.begin(SD_CS, SD_SPI, 40000000))
  {
    Serial.println("Card Mount Failed");
    return 1;
  }
  else
  {
    Serial.println("Card Mount Successed");
  }
  listDir(SD, "/", 0);
  Serial.println("TF Card init over.");
  return 0;
}

// SD test
void listDir(fs::FS & fs, const char *dirname, uint8_t levels)
{
  Serial.printf("Listing directory: %s\n", dirname);

  File root = fs.open(dirname);
  if (!root)
  {
    Serial.println("Failed to open directory");
    return;
  }
  if (!root.isDirectory())
  {
    Serial.println("Not a directory");
    return;
  }

  File file = root.openNextFile();
  while (file)
  {
    if (file.isDirectory())
    {
      Serial.print("  DIR : ");
      Serial.println(file.name());
      if (levels)
      {
        listDir(fs, file.name(), levels - 1);
      }
    }
    else
    {
      Serial.print("  FILE: ");
      Serial.print(file.name());
      Serial.print("  SIZE: ");
      Serial.println(file.size());
    }
    file = root.openNextFile();
  }
}

//Camera setting
void camera_init()
{
  //  // camera config
  //  camera_config_t config;
  //  config.ledc_channel = LEDC_CHANNEL_0;
  //  config.ledc_timer = LEDC_TIMER_0;
  //  config.pin_d0 = Y2_GPIO_NUM;
  //  config.pin_d1 = Y3_GPIO_NUM;
  //  config.pin_d2 = Y4_GPIO_NUM;
  //  config.pin_d3 = Y5_GPIO_NUM;
  //  config.pin_d4 = Y6_GPIO_NUM;
  //  config.pin_d5 = Y7_GPIO_NUM;
  //  config.pin_d6 = Y8_GPIO_NUM;
  //  config.pin_d7 = Y9_GPIO_NUM;
  //  config.pin_xclk = XCLK_GPIO_NUM;
  //  config.pin_pclk = PCLK_GPIO_NUM;
  //  config.pin_vsync = VSYNC_GPIO_NUM;
  //  config.pin_href = HREF_GPIO_NUM;
  //  config.pin_sscb_sda = SIOD_GPIO_NUM;
  //  config.pin_sscb_scl = SIOC_GPIO_NUM;
  //  config.pin_pwdn = PWDN_GPIO_NUM;
  //  config.pin_reset = RESET_GPIO_NUM;
  //  config.xclk_freq_hz = 20000000;
  //
  //  config.pixel_format = PIXFORMAT_RGB565;
  //  config.frame_size = FRAMESIZE_QVGA;
  //  //  config.frame_size = FRAMESIZE_UXGA;
  //  config.jpeg_quality = 10;
  //  config.fb_count = 1;
  // camera config
  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;

  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;
  //  config.grab_mode = CAMERA_GRAB_LATEST;
  config.pixel_format = PIXFORMAT_RGB565;
  //  config.frame_size = FRAMESIZE_QVGA;
  config.frame_size = FRAMESIZE_HVGA;

  config.jpeg_quality = 10;
  config.fb_count = 4;
  // camera init
  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK)
  {
    Serial.printf("Camera init failed with error 0x%x", err);
    while (1)
      ;
  }

  sensor_t *s = esp_camera_sensor_get();
  // initial sensors are flipped vertically and colors are a bit saturated
  if (s->id.PID == OV2640_PID)
  {
    s->set_vflip(s, 0);   // vertical flip
    s->set_hmirror(s, 1); // Horizontal mirror

    s->set_brightness(s, 0); // up the blightness just a bit
    s->set_saturation(s, 1); // lower the saturation
  }
  // drop down frame size for higher initial frame rate
  s->set_framesize(s, FRAMESIZE_HVGA);

  //show_log(0);
}

// Save image to SD card
int save_image(fs::FS &fs, uint8_t *rgb)
{
  imgname = "/" + String(img_index) + "write.bmp";
  img_index++;

  Serial.println("Image name：" + imgname);

  File f = fs.open(imgname, "w");
  if (!f)
  {
    Serial.println("Failed to open file for writing");
    return -1;
  }

  f.write(img_rgb888_480_320_head, 54);
  f.write(rgb, 480 * 320 * 3);
  f.close();
  return 0;
}
